function [yout,zout,dens] = vo_vdrad(fov,nsize,acc,ncal,tl,numph,param)
% VO_VDRAD accelerated radial view ordering
%   [YO,ZO] = VO_VDRAD(FOV,SIZE,ACC,NCAL,TL,NUMPH,[PARAM])
%     FOV   -- [mm]    [FOVy FOVz] 
%     NSIZE -- [int]   [ny nz] size of acq matrix
%     ACC   -- [float] [Ry Rz] acceleration factor
%     NCAL  -- [int]   size of calibration area
%     TL    -- [int]   train length of each spoke
%     NUMPH -- [int]   num phases to output -- one phase is defined as when
%                      the calibration area is fully sampled and the desired
%                      acceleration is achieved.  (default = 1)
%     PARAM -- [CELL]  optional argument specifying different features:
%        'type'      -- [string] shape of each ring.
%                                options: 'ellipse','square','triangle'
%                                  (default = 'ellipse')
%        'mode'      -- [int]    ordering of each spoke
%                                options: 1-out to center, 
%                                         2-center out and back, 
%                                         otherwise-center to out
%                                  (default = 0)
%        'dofib'     -- [int]    round the smallest dofib rings to the 
%                                nearest Fibonacci number. This covers the 
%                                rings as best as possible for 1 temporal
%                                phase at the cost of possibly re-sampling
%                                  (default = 1)
%        'corner'    -- [int]    if 1, corner cut k-space 
%                                  (default = 1)
%        'cornercal' -- [int]    if 1, corner cut k-space in calib region 
%                                  (default = 1)
%        'spiral'    -- [int]    adds a spiral twist to each radial spoke
%                                  (default = 10)    
%        'dbprint'   -- [int]    if 1, verbose print outs
%                                  (default = 0)
%        'demo'      -- [int]    if>0, plot figure with figure
%                                number specified by the argument
%                                  (default = 0)
%
% Examples:
%   [yo,zo] = VO_VDRAD([300 300],[256 256],...
%                           [4 2],24,24,1,{'type','square'});
%   [yo,zo] = VO_VDRAD([300 200],[256 256],...
%                           [4 2],24,24,1,{'dofib','4'});
%
% See also VO_DRAWORDERTRAIN.
%
% (c) Joseph Y Cheng (jycheng@mrsrl.stanford.edu) 2013
%   
    
% Log: 
%   - Updated (2013 Feb 28 jycheng).
%   - Added 'demo' feature (2013 Jun 7 jycheng)
%
% SVN info: 
%   Date:     $Date: 2013-07-23 15:41:23 -0700 (Tue, 23 Jul 2013) $
%   Revision: $Revision: 1349 $
%   Author:   $Author: jycheng $
%   Id:       $Id: vo_vdrad.m 1349 2013-07-23 22:41:23Z jycheng $

        
    if nargin<6
        numph = 1;
    end
    if nargin<7
        param = [];
    end

    %% Extract parameters
    p = parseparam(param);
    shape     = p.type;
    fib       = p.dofib;
    cornercut = p.corner;
    ccutcal   = p.cornercal;
    spiral    = p.spiral;
    mode      = p.mode;
    vdrate    = p.vdrate;
    DBPLOT    = p.dbplot;
    DBPRINT   = p.dbprint;
    DEMO      = p.demo;
    
    %% Check some params
    if fib<0
        fib = 0;
    elseif fib>tl-1
        fib = tl-1;
    end
    if spiral<0
        spiral = 0;
    end
    if DEMO<0
        DEMO = 0;
    end
    
    %% Init output
    yout = []; zout = [];
    
    %% FOV [mm]
    fovy = fov(1);
    fovz = fov(2);
    
    %% Num of points
    ny = nsize(1);
    nz = nsize(2);
    
    %% Acceleration
    ry = (acc(1));
    rz = (acc(2));
    if nz==1
        rz = 1;
    end
    if ry*rz<1
        dbdisp(sprintf('ERROR! Invalid acceleration: (%g,%g)',ry,rz));
        return;
    end

    %% Acceleration setup
    rr = ry/rz; %% ratio
        
    %% Find initial ordering (based on seq)
    yi = repmat((1:ny),[nz 1]);
    zi = repmat((1:nz).',[1 ny]);
    yi = yi(:); zi = zi(:);
    
    %% Convert ordering to k-space coordinate points
    ky = -1+2*(yi-.5)/ny;
    if nz>1
        kz = -1+2*(zi-.5)/nz;
    else
        kz = 0*zi;
    end
    
    %% Corner cut (based on Miki's code)
    if cornercut
        dny = 2;
        yelledge = -1+2*(floor(ny/2)-1-dny)/(ny-1);
        ellmin = sqrt(1+yelledge^2);
        ndec = abs(floor(log10(ellmin-1)));        
        ellmin = 1+ceil((10^ndec)*(ellmin-1)+0.5)/10^ndec;
        ind = find((ky.^2+kz.^2)<=ellmin);
        yi = yi(ind); zi = zi(ind);
        ky = ky(ind); kz = kz(ind);
        clear va ind;
    end
    
    npe = length(yi(:));
    
    %% Find calib area (based on Miki's code)
    %%   make sure same resolution in both directions
    %% Adjust scaling to get same extent in k-space in 
    %% bother directions
    %% resolution [mm] (1/fovmax)
    res = fov./nsize; 
    %% resolution ratio
    rs = [1 1];
    %% find largest res
    if res(1)>res(2)
        rs(1) = res(2)/res(1);
    else
        rs(2) = res(1)/res(2);
    end
    
    %% Adjust for acceleration
    rry = 1; rrz = 1;
    if rz>ry
        rry = (ry/rz);
    else
        rrz = (rz/ry);
    end
    %% Convert to complex space for easier calc
    %%   adjusted (ky,kz) for acceleration and spokes
    kadj = rs(1)*ky*rry + i*rs(2)*kz*rrz; 
    %%   actual (ky,kz) for finding calib region
    kact = rs(1)*ky + i*rs(2)*kz;
    
    %% k-Space magnitude (k_r) based on adjusted
    switch shape
     case 1 %% square
      va = max(abs(real(kadj(:))),abs(imag(kadj(:))));
     case 2
      va = abs(real(kadj(:)))+abs(imag(kadj(:)));
     otherwise %% ellipse
      va = abs(kadj(:));
    end
    %% k-Space angle (k_\theta) based on adjusted
    vp = angle((kadj(:)).*exp(i*spiral*va));

    if DBPLOT
        sampa = zeros(nsize);
        sampp = zeros(nsize);
        for N=1:length(kadj)
            sampa(yi(N),zi(N)) = va(N);
            sampp(yi(N),zi(N)) = vp(N);
        end
        maxa = max(va(:));
        mina = min(va(:));
        sfigure(DBPLOT);
        subplot(121),imshow(sampa,[mina+0.1 maxa]),title('radius (k_r)');
        colorbar;
        subplot(122),imshow(sampp,[]),title('angle (k_\theta)'),colorbar;
    end
    
    %% [adjusted-phase, adjusted-mag, index]
    v    = [vp,va,(1:length(kadj(:))).'];  
    %% [actual-phase, actual-mag, index]
    vact = [angle(kact(:)),abs(kact(:)),(1:length(kact(:))).'];
    
    %% Adjust calibration region size to fit desired acceleration
    ncal_i = ncal;
    ncalmin = 10;   
    numcalib = npe*2;
    while ry*rz>npe/numcalib
        %% calculate calibration area
        %% r0 -- ratio of num calib to largest dim
        if ny>nz
            r0 = ncal_i/ny;
        else
            r0 = ncal_i/nz;
        end
        %% rs(1)*abs(ky(:))<r0 ---> 
        %%     if fovz/fovy*ny/nz<1,  
        %%                        fovz/fovy*ny/nz*ky < ncal/nmax
        %%                                        kz < ncal/nmax
        %%     else,                              ky < ncal/nmax
        %%                        fovy/fovz*nz/ny*kz < ncal/nmax
        if ccutcal        
            maskcalib = sqrt((rs(1)*ky(:)).^2+(rs(2)*kz(:)).^2)<r0;
        else
            maskcalib = (abs(rs(1)*ky(:))<r0).*(abs(rs(2)*kz(:))<r0);
        end
        numcalib = sum(maskcalib(:));
        
        if ry*rz>npe/numcalib
            dbdisp('WARNING! acceleration too great for calib size');
            dbdisp(sprintf('  Adjusting calib size from %d to %d',...
                           ncal_i,ncal_i-1));
            ncal_i = ncal_i-1;
            if ncal_i<ncalmin
                dbdisp('ERROR! Unable to find sampling pattern');
                dbdisp(sprintf('  Acc = (%g,%g)',ry,rz));
                return;
            end
        end
    end
        
    %% tl == number of rings
    %% number of spokes for one phase 
    %%  (also, number of spokes to fully sample the calib region)
    if fib            
        nspoke0 = floorfib(npe/(ry*rz*tl));
    else
        nspoke0 = round(npe/(ry*rz*tl));
    end
    lspoke0 = ceil(numcalib/nspoke0);
    nring = tl-lspoke0+1;
    
    lspoke = [lspoke0,ones(1,nring-1)];
    if ry*rz <= 1
        ag = 1;
        nspoke = ones(1,nring)*nspoke0;
        %% Adjust last ring to make sure we get the right num of points
        nspoke(end) = npe-sum(nspoke.*lspoke)+nspoke(end);
    else            
        lspoke = [lspoke0,ones(1,nring-1)];
        %% Find variability
        ag = (npe-nspoke0*tl)/sum([1:(nring-1)].^vdrate);
        %% Compute num of views per ring
        nspoke = [nspoke0,[(1:(nring-1)).^vdrate*ag+nspoke0]];
        for N=1:(fib-1)
            nspoke(N+1) = roundfib(nspoke(N+1));
        end
        nspoke = round(nspoke);
        
        %% Adjust last ring to make sure we get the right num of points
        nspoke(end) = npe-sum(nspoke.*lspoke)+nspoke(end);
    end
    %if DBPLOT
    %    sfigure(DBPLOT+1),plot(nspoke),title('N_{views}');
    %end
    
    act_acc_n = npe/sum(nspoke(1)*lspoke(:));    
    lring = nspoke.*lspoke;
    
    if DBPRINT
        dbdisp('---------------------------------------');
        dbdisp('VIEW ORDERING: ACCELERATED RADIAL');
        dbdisp('---------------------------------------');
        dbdisp(sprintf('  Num views:             %d',npe));
        dbdisp(sprintf('  Num rings:             %d',nring));
        dbdisp(sprintf('  Desired num calib pts: %d',numcalib));
        dbdisp(sprintf('  Actual num calib pts:  %d',nspoke(1)*lspoke(1)));
        dbdisp(sprintf('  Desired ncal:          %d',ncal));
        dbdisp(sprintf('  Actual ncal:           %d',ncal_i));
        dbdisp(sprintf('  Num calib spokes:      %d',lspoke0));
        dbdisp(sprintf('  Num spokes for calib: ~%d',nspoke(1)));
        dbdisp(sprintf('  Num fib rings:         %d',fib));
        dbdisp(sprintf('  Spoke length:          %d',sum(lspoke(:))));
        dbdisp(sprintf('  Mode:                  %d',mode));
        dbdisp(sprintf('  Spiral twist:          %d',spiral));
        dbdisp(sprintf('  Corner cut?            %d',cornercut));
        dbdisp(sprintf('  Acc grad factor:       %g',ag));
        dbdisp(sprintf('  VD rate:               %g',vdrate));
        dbdisp('');
        dbdisp('');        
    end
    if DBPLOT || DEMO
        ringmask = zeros(nsize);
    end
    
    %% First find calibration area...
    if ccutcal
        vact = sortrows(vact,2); %% Sort by actual radius
        vc = v(vact(1:(nspoke(1)*lspoke(1)),end),:);
        vo = v(vact((1+nspoke(1)*lspoke(1)):end,end),:);    
        vo = sortrows(vo,2); % sort by radius
    else
        vo_c = v(vact(find(1-maskcalib),end),:);
        vo_c = sortrows(vo_c,2); %% Sort by radius
        vc = v([vact(find(maskcalib),end); ...
                vo_c(1:(nspoke(1)*lspoke0-numcalib),end)],:);
        vo = v(vo_c((1+(nspoke(1)*lspoke0-numcalib)):end,end),:);        
        vo = sortrows(vo,2);
    end
    
    %% Build ring table
    Rtab = []; rout = [];
    for R=1:nring
        if R==1
            vi = vc;
        else
            indr = (1:lring(R))+sum(lring(2:(R-1)));
            vi = vo(indr,:);
        end
        vi = sortrows(vi,1); % sort by jittered-adj-angle
        ri = [];
        ri = [yi(vi(:,end)),zi(vi(:,end))];
        Rtab{R} = vi;
        
        if nargout==4
           ri = [yi(vi(:,end)),zi(vi(:,end))];
           rout{R} = ri;
        end
        if DBPLOT || DEMO
            if mod(R,2)
                scalei = 0.5;
            else
                scalei = 1;
            end
            for N=1:size(vi,1)
                ringmask(yi(vi(N,end)),zi(vi(N,end))) = scalei;
            end
        end
    end
    
    mask_act = zeros(nsize);
    I_ph = 0;                    %% index for current phase 
    rph  = 1;                    %% phase acceleration (fractional phases)
    Rang = zeros(nring,1);       %% angle for current index for each ring
    Aph  = zeros(ceil(numph),1); %% actual acceleration
    Nph  = zeros(ceil(numph),1); %% actual num-spokes per phase                    
    ytab = []; ztab = [];        %% tab (y,z)-indeces: (I_spoke,I_ring)
    I_spoke = 1;
    while I_ph < numph
        for R=1:nring
            %% Indeces from table
            vi = Rtab{R};
                    
            %% Increment by golden-angle (~137.5 deg)
            [I_vi,R_vi] = inc_golden(Rang(R),nspoke(R));
            Rang(R) = R_vi;
            
            %% Take caution if lspoke(1)>1
            I_ring = (1:lspoke(R))+sum(lspoke(1:(R-1)));
            I_vi   = (1:lspoke(R))+(I_vi-1)*lspoke(R);
            %% Unwrap indeces to be <= nspoke(R)
            ind_check = find(I_vi>nspoke(R)*lspoke(R));
            I_vi(ind_check) = I_vi(ind_check)-nspoke(R)*lspoke(R);
            
            %% Add to table
            ytab(I_spoke,I_ring) = yi(vi(I_vi,end));
            ztab(I_spoke,I_ring) = zi(vi(I_vi,end));
        end
        
        %% Add spoke to acquisition mask 
        for N=1:tl
            mask_act(ytab(I_spoke,N),ztab(I_spoke,N)) = 1;
        end
        %% Adjust for fractional phase
        if (numph-I_ph)<=numph-floor(numph)
            rph = numph-floor(numph);
        end
        %% Check acceleration 
        Aph(I_ph+1) = npe/sum(mask_act(:));
        if Aph(I_ph+1)*rph<ry*rz
            mask_act = zeros(nsize);
            Nph(I_ph+1) = I_spoke-sum(Nph(1:(I_ph+1)));
            I_ph = I_ph+1;
        end
        I_spoke = I_spoke+1;
    end
    ntotspoke = I_spoke-1;
    
    if DBPRINT
        dbdisp(sprintf('  Total num spokes:      %d',ntotspoke));
        for N=1:length(Aph)
            dbdisp(sprintf('    [Phase %d]: Act acc:     %g',N,Aph(N)));
            dbdisp(sprintf('    [Phase %d]: Num spokes:  %g',N,Nph(N)));
        end
    end
    if DBPLOT
        sfigure(DBPLOT+2),imshow(ringmask,[0 1]),title('Ring division');
    end
    
    %% Finalize
    %% Sort each train
    ky = rs(1)*(-1+2*(ytab-1)/ny);
    kz = rs(2)*(-1+2*(ztab-1)/nz);
    kr = abs(ky+i*kz);
    yout = []; zout = [];
    for I=1:ntotspoke
        vi = [kr(I,1:tl).',(1:tl).'];
        vi = sortrows(vi,1);
        switch mode
         case 1
          yout = [yout, ytab(I,vi(end:-1:1,end))];
          zout = [zout, ztab(I,vi(end:-1:1,end))];
         case 2
          i1 = vi(1:2:end,end);
          i2 = vi(2:2:end,end);
          i2 = i2(end:-1:1);
          yout = [yout, ytab(I,i1), ytab(I,i2)];
          zout = [zout, ztab(I,i1), ztab(I,i2)];
         otherwise
          yout = [yout, ytab(I,vi(:,end))];
          zout = [zout, ztab(I,vi(:,end))];
        end
    end
    
    densv = [];
    for N=1:length(nspoke)
        densv = [densv nspoke(N)*ones(1,lspoke(N))];
    end
    
    dens = sqrt(repmat(densv/densv(1),[1 ntotspoke]));
     
    mask_ph1 = zeros(nsize);
    mask_all = zeros(nsize);
    for N=1:Nph(1)*tl
        mask_ph1(yout(N),zout(N)) = 1;
    end
    for N=1:ntotspoke*tl
        mask_all(yout(N),zout(N)) = 1;
    end
    if DBPRINT
        dbdisp(sprintf('  Combined acc factor:   %g',npe/sum(mask_all(:))));
    end
    
    if DBPLOT
        sfigure(DBPLOT+3); imshow(mask_ph1,[]); title('Sampling mask');
        %sfigure(DBPLOT+4); imshow(mask_all,[]);
        scale = sum(mask_ph1(:));
        sfigure(DBPLOT+5); imshow(abs(ifftnc(mask_ph1)),[0 1e2/scale]);
        title('Undersampling PSF');
    end
    
    if DEMO
        sampa = zeros(nsize);
        sampp = zeros(nsize);
        for N=1:length(kadj)
            sampa(yi(N),zi(N)) = va(N);
            sampp(yi(N),zi(N)) = vp(N);
        end
        maxa = max(va(:));
        mina = min(va(:));
        sfigure(DEMO);
        
        subplot(231);
        imshow(sampa,[mina+0.1 maxa]),title('radius (k_r)');%,colorbar;        
        subplot(232);
        imshow(sampp,[]),title('angle (k_\theta)');%,colorbar;       
        subplot(233);
        imshow(ringmask,[0 1]),title('Ring division');
        subplot(234);
        imshow(mask_ph1,[0 1]); title('Sampling mask');
        subplot(235); scale = sum(mask_ph1(:));
        imshow(abs(ifftnc(mask_ph1)),[0 1e2/scale]),title('Undersampling PSF');
        drawnow;
    end
    
    return;

function [I,A] = inc_golden(Ac,N)
% INC_GOLDEN increment index using golden cut
%   [I,A] = INC_GOLDEN(Ac,N)
%     Ac -- current angle
%     N  -- max index
%     I  -- next index
%     A  -- next angle
    
    %% golden ratio
    gr = (sqrt(5)-1)/2;
    %% golden angle
    gra = 360*gr;
    %% delta angle
    da = 360/N;
    
    %% increment angle (unwrap if needed)
    A = Ac+gra;
    if A>360
        A = A-360;
    end
    %% find closest index
    I = round(A/da)+1;
    
function p = parseparam(paramcell)
    
    if mod(size(paramcell),2) ~= 0
        dbdisp('Invalid parameters!');
        p = -1;
        return;
    end
    
    %% default
    p = [];
    p.type      = 0;
    p.mode      = 0;
    p.corner    = 1;
    p.dofib     = 1;
    p.cornercal = 1;
    p.spiral    = 10;
    p.dbprint   = 0;
    p.dbplot    = 0;
    p.demo      = 0;
    p.vdrate    = 1;
    
    %% look through user defined params
    for N=1:(size(paramcell,2)/2)
        pname = paramcell{2*N-1};
        parg  = paramcell{2*N};
        switch pname
         case 'type'
          switch parg
           case 'ellipse'
            p.type = 0;
           case 'square'
            p.type = 1;
           case 'triangle'
            p.type = 2;
           otherwise
            dbdisp(sprintf('Invalid type %s, set to default.',parg));
            p.type = 0;
          end
         case 'mode'
          p.mode = parg;
         case 'corner'
          p.corner = parg;
         case 'dofib'
          p.dofib = parg;
         case 'cornercal'
          p.cornercal = parg;
         case 'spiral'
          p.spiral = parg;
         case 'vdrate'
          p.vdrate = parg;       
         case 'dbplot'
          p.dbplot = parg;
         case 'dbprint'
          p.dbprint = parg;
         case 'demo'
          p.demo = parg;
         otherwise
          dbdisp(sprintf('Ignoring ivalid parameter %s.',pname));
        end
    end